import { EditorView } from '@codemirror/view';

const docChangeListener = (handler) => {
    return EditorView.updateListener.of((update) => {
        if (!update.docChanged)
            return;
        const { state } = update.view;
        handler(update.view, state.doc.sliceString(0));
    });
};
function onDocChange(handler) {
    return docChangeListener(handler);
}

const selectionChangeListener = (handler) => {
    let timer;
    let first = true;
    return EditorView.updateListener.of((v) => {
        if (!v.selectionSet)
            return;
        if (v.startState.selection.eq(v.state.selection)) {
            return;
        }
        timer && clearTimeout(timer);
        timer = window.setTimeout(() => {
            if (first) {
                first = false;
            }
            handler(v.view, v.state.selection.ranges.map((r) => ({
                from: r.from,
                to: r.to
            })));
        }, first ? 0 : 100);
    });
};
function onSelectionChange(handler) {
    return selectionChangeListener(handler);
}

export { onDocChange, onSelectionChange };
